@@ -87,17 +87,19 @@ def _warn_when_local_changes(func: Callable) -> Callable:
 
     @functools.wraps(func)
     def wrapper(self, *args, **kwargs):
-        localdiff = self.get_diff()
-        if (
-            localdiff[constants.DELETED_FILES]
-            or localdiff[constants.MODIFIED_FILES]
-            or localdiff[constants.NEW_FILES]
-        ) and not kwargs.get("forced", False):
-            logger.warning(
-                "Local changes detected! Please run stash() to stash your local changes, "
-                "or run this function again with forced=True"
-            )
-            return None
+        # only do the following checking when HEAD file exist(dataset exists on local host)
+        if os.path.exists(self._get_file_path(constants.DATASET_HEAD_FILE)):
+            localdiff = self.get_diff()
+            if (
+                localdiff[constants.DELETED_FILES]
+                or localdiff[constants.MODIFIED_FILES]
+                or localdiff[constants.NEW_FILES]
+            ) and not kwargs.get("forced", False):
+                logger.warning(
+                    "Local changes detected! Please run stash() to stash your local changes, "
+                    "or run this function again with forced=True"
+                )
+                return None
 
         return func(self, *args, **kwargs)
 
