@@ -26,7 +26,12 @@ from gantry.logger.event_builder import (
     _create_timestamp_idx,
 )
 from gantry.logger.stores import APILogStore
-from gantry.logger.types import DataLink
+from gantry.logger.types import (
+    DataLink,
+    IngestFromDataConnectorRequest,
+    IngestFromDataConnectorResponse,
+    Schedule,
+)
 from gantry.logger.utils import (
     _batch_fail_msg,
     _batch_success_msg,
@@ -279,6 +284,126 @@ class Gantry:
 
         return batch_id
 
+    @_log_exception
+    @typechecked
+    def log_from_data_connector(
+        self,
+        application: str,
+        source_data_connector_name: str,
+        timestamp: Optional[str] = None,
+        inputs: Optional[List[str]] = None,
+        outputs: Optional[List[str]] = None,
+        feedbacks: Optional[List[str]] = None,
+        join_key: Optional[str] = None,
+        row_tags: Optional[List[str]] = None,
+        global_tags: Optional[Dict[str, str]] = None,
+        schedule: Optional[Schedule] = None,
+        pipeline_name: Optional[str] = None,
+    ) -> IngestFromDataConnectorResponse:
+        """
+        Function to ingest source tabular records from a registered source data
+        connector into Gantry.
+
+        To log predictions using this function, both column names of inputs and outputs
+        must be passed.
+        To log feedback using this function, both column names of join_key and feedback
+        must be passed.
+
+        Example:
+
+        .. code-block:: python
+
+            # Record an application's predictions.
+            gantry.log_from_data_connector(
+                application="foobar",
+                source_data_connector_name="my_snowflake_connector",
+                inputs=["column_A", "column_B"],
+                outputs=["column_C"],
+                timestamp="column_T",
+                global_tags = {"env":"dev", "version": "1.0.0"},
+                row_tags=["column_D"],
+                join_key="column_E",
+            )
+
+            # Record an application's feedbacks.
+            # to a previously ingested prediction.
+            gantry.log_from_data_connector(
+                application="foobar",
+                source_data_connector_name="my_snowflake_connector",
+                feedbacks=["column_E", "column_F"],
+                timestamp="column_T",
+                global_tags = {"env":"dev", "version": "1.0.0"},
+                row_tags=["column_D"],
+                join_key="column_E",
+            )
+
+            # Trigger scheduled ingestion every 8 hours from a data connector incrementally.
+            from gantry.logger.types import (
+                Schedule,
+                ScheduleFrequency,
+                ScheduleType,
+                ScheduleOptions,
+            )
+
+            gantry.log_from_data_connector(
+                application="foobar",
+                source_data_connector_name="my_snowflake_connector",
+                inputs=["column_A", "column_B"],
+                outputs=["column_C"],
+                timestamp="column_T",
+                global_tags = {"env":"dev", "version": "1.0.0"},
+                row_tags=["column_D"],
+                join_key="column_E",
+                schedule=Schedule(
+                    start_on="2023-01-14T17:00:00.000000",
+                    frequency=ScheduleFrequency.EVERY_8_HOURS,
+                    type=ScheduleType.INCREMENTAL_APPEND,
+                    options=ScheduleOptions(watermark_key="column_T"),
+                )
+            )
+
+        Args:
+            application (str): Name of the application to ingest data into
+            source_data_connector_name (str): Name of the registered source data connector
+                to ingest data from.
+            timestamp (optional, str): by default, the timestamp values will be filled
+                with the timestamp at ingestion. Set this parameter to reference a
+                different column.
+            inputs (optional, list[str]): A list of column names for inputs.
+            outputs (optional, list[str]): A list ofcolumn names for outputs.
+            feedbacks (optional, list[str]): A list of column names for feedbacks.
+            join_key (optional, str): A column name to use as the join key to identify each
+                record.
+            row_tags (optional, list[str]): A list of column names to use the values of
+                as tags for each row.
+            global_tags (optional, list[dict[str, str]]): A list of dictionaries of string
+                key and value pairs to tag the entire ingestion from this data connector
+            schedule (optional, Schedule): An optional parameter to schedule the ingestion.
+            pipeline_name (optional, str): An optional parameter to specify the ingestion
+                pipeline's name
+        """
+        if schedule and schedule.start_on and isinstance(schedule.start_on, datetime.datetime):
+            schedule.start_on = schedule.start_on.isoformat()
+
+        request = IngestFromDataConnectorRequest(
+            application=application,
+            source_data_connector_name=source_data_connector_name,
+            timestamp=timestamp,
+            inputs=inputs,
+            outputs=outputs,
+            feedbacks=feedbacks,
+            row_tags=row_tags,
+            global_tags=global_tags,
+            join_key=join_key,
+            schedule=schedule,
+            pipeline_name=pipeline_name,
+        )
+
+        response = self.log_store.log_from_data_connector_async(request=request)
+        logger.info("Successfully submitted ingestion request to Gantry.")
+
+        return response
+
     @_log_exception
     @typechecked
     def log_records(  # noqa: C901
