@@ -16,6 +16,7 @@ from gantry.const import DEFAULT_ENV, PROD_API_URL
 from gantry.exceptions import ClientNotInitialized
 from gantry.logger.client import Gantry
 from gantry.logger.stores import APILogStore
+from gantry.logger.types import IngestFromDataConnectorResponse, Schedule
 
 colorama.init(autoreset=True)
 logger_obj = logging.getLogger(__name__)
@@ -169,6 +170,23 @@ def log_file(
     return _CLIENT.log_file(**locals())  # type: ignore[union-attr]
 
 
+@_client_alias
+def log_from_data_connector(
+    application: str,
+    source_data_connector_name: str,
+    timestamp: Optional[str] = None,
+    inputs: Optional[List[str]] = None,
+    outputs: Optional[List[str]] = None,
+    feedbacks: Optional[List[str]] = None,
+    row_tags: Optional[List[str]] = None,
+    global_tags: Optional[Dict[str, str]] = None,
+    join_key: Optional[str] = None,
+    schedule: Optional[Schedule] = None,
+    pipeline_name: Optional[str] = None,
+) -> IngestFromDataConnectorResponse:
+    return _CLIENT.log_from_data_connector(**locals())  # type: ignore[union-attr]
+
+
 @_client_alias
 def log_record(
     application: str,
