@@ -2,6 +2,7 @@ import atexit
 import json
 import logging
 import logging.handlers
+from dataclasses import asdict
 from queue import Queue
 from typing import Callable, Collection, Dict, Iterator, List, Optional
 
@@ -11,6 +12,10 @@ import requests
 from gantry.api_client import APIClient
 from gantry.exceptions import GantryRequestException
 from gantry.logger.consumer import BatchConsumer
+from gantry.logger.types import (
+    IngestFromDataConnectorRequest,
+    IngestFromDataConnectorResponse,
+)
 from gantry.serializers import EventEncoder
 
 logger = logging.getLogger(__name__)
@@ -205,6 +210,19 @@ class APILogStore(BaseLogStore):
             for e in events:
                 self.queue.put(e)
 
+    def log_from_data_connector_async(
+        self, request: IngestFromDataConnectorRequest
+    ) -> IngestFromDataConnectorResponse:
+        response = self._api_client.request(
+            "POST",
+            "/api/v1/ingest/data-connectors",
+            json=asdict(request),
+            headers={"Content-Type": "application/json"},
+            raise_for_status=True,
+        )
+
+        return IngestFromDataConnectorResponse(**response["data"])
+
     def _join(self):
         """
         Ends the consumer thread once the queue is empty.
