@@ -1,5 +1,5 @@
 from gantry.query.core.dataframe import GantrySeries
-from gantry.query.core.utils import check_response, same_size
+from gantry.query.core.utils import same_size
 
 
 class GantryDistance(object):
@@ -81,8 +81,9 @@ class GantryDistance(object):
                 },
             }
         }
-        response = self._api_client.request("POST", "/api/v1/aggregate/query", json=data)
-        check_response(response)
+        response = self._api_client.request(
+            "POST", "/api/v1/aggregate/query", json=data, raise_for_status=True
+        )
 
         try:
             return response["data"]["query"][stat]
