@@ -12,6 +12,7 @@ import sys
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Set, Tuple
 
+import click
 import requests
 from dateutil import parser
 
@@ -278,3 +279,11 @@ def pprint_records(records: List[Dict[str, str]], include_keys: Optional[Set[str
         pp.pprint(
             [dict([(k, v) for k, v in record.items() if k in include_keys]) for record in records]
         )
+
+
+def check_response_status(resp, error_prefix: str):
+    if resp.get("response") != "ok":
+        click.secho("FAILED.", fg="red")
+        err_msg = resp.get("error", "unknown error")
+        raise click.ClickException(f"{error_prefix}: {err_msg}.")
+    click.secho("SUCCESS", fg="cyan")
