@@ -1,4 +1,7 @@
 import json
+import logging
+import os
+import uuid
 from io import StringIO
 from pathlib import Path
 
@@ -10,16 +13,23 @@ from mock import patch
 from responses import matchers
 
 from gantry.api_client import APIClient
-from gantry.utils import list_all_files
-from gantry.dataset.constants import DELETED_FILES, MODIFIED_FILES, NEW_FILES
+from gantry.dataset.constants import (
+    DATASET_MANIFEST_FILE,
+    DATASET_STASH_FILE,
+    DELETED_FILES,
+    MODIFIED_FILES,
+    NEW_FILES,
+    STASH_FOLDER,
+)
 from gantry.dataset.gantry_dataset import DatasetFileInfo, GantryDataset
 from gantry.exceptions import (
     DatasetCommitNotFoundError,
     DatasetHeadOutOfDateException,
+    DatasetNotFoundError,
     GantryException,
     GantryRequestException,
-    DatasetNotFoundError,
 )
+from gantry.utils import get_files_checksum, list_all_files
 
 from .conftest import (
     AWS_REGION,
@@ -91,7 +101,7 @@ def test_get_commit(gantry_dataset_obj, commit_history):
             },
             headers={"Content-Type": "application/json"},
         )
-        commit = gantry_dataset_obj.get_commit(expected_commit["id"])
+        commit = gantry_dataset_obj._get_commit(expected_commit["id"])
         assert commit == expected_commit
 
 
@@ -109,7 +119,7 @@ def test_get_commit_error(gantry_dataset_obj, commit_history, status_code, expec
             headers={"Content-Type": "application/json"},
         )
         with pytest.raises(expected_error):
-            gantry_dataset_obj.get_commit(commit_id)
+            gantry_dataset_obj._get_commit(commit_id)
 
 
 def test_get_latest_commit(gantry_dataset_obj, commit_history):
@@ -126,7 +136,7 @@ def test_get_latest_commit(gantry_dataset_obj, commit_history):
             },
             headers={"Content-Type": "application/json"},
         )
-        latest_commit = gantry_dataset_obj.get_latest_commit()
+        latest_commit = gantry_dataset_obj._get_latest_commit()
         assert latest_commit["id"] == "8ee0f6d5-c84c-473a-931a-5148b2e704d7"
         assert latest_commit["is_latest_commit"]
 
@@ -457,7 +467,7 @@ def create_mock_manifest(commit_history):
 
 
 @pytest.mark.parametrize("commit_id", [None, "eb2e7242-3340-4edf-8366-90d4fce897ce"])
-def test_sync_dataset(commit_id, datadir, gantry_dataset_obj, commit_history):
+def test_pull_dataset(commit_id, datadir, gantry_dataset_obj, commit_history):
     """
     Test sync local data set
     1. To latest commit
@@ -471,8 +481,8 @@ def test_sync_dataset(commit_id, datadir, gantry_dataset_obj, commit_history):
         setup_resp_for_deleted_check(resp)
         setup_resp_for_sync(resp, commit_id, commit_json, manifest_file)
 
-        commit = gantry_dataset_obj.pull(commit_id)
-        assert commit == commit_json
+        commit = gantry_dataset_obj.pull(commit_id, forced=True)
+        assert commit == gantry_dataset_obj._prune_commit_info(commit_json)
         with open(datadir / "show_diff_dataset/dataset_config.yaml", "r") as f:
             assert f.read() == "fake_response"
 
@@ -486,6 +496,23 @@ def test_sync_dataset(commit_id, datadir, gantry_dataset_obj, commit_history):
         assert not list_all_files(Path(datadir / "show_diff_dataset/artifacts"))
 
 
+def test_pull_local_changes(datadir, gantry_dataset_obj):
+    """
+    Test trying to pull when local changes exist without forced=True
+    """
+    with open(datadir / "show_diff_dataset/.dataset_metadata/HEAD") as f:
+        original_head = json.load(f)
+
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp)
+        commit = gantry_dataset_obj.pull()
+
+        assert commit is None
+
+        with open(datadir / "show_diff_dataset/.dataset_metadata/HEAD") as f:
+            assert original_head == json.load(f)
+
+
 def test_to_jsonl():
     file_info = DatasetFileInfo(
         file_name="test_file",
@@ -544,7 +571,7 @@ def test_rollback(datadir, gantry_dataset_obj, commit_history):
         setup_resp_for_deleted_check(resp)
         setup_resp_for_sync(resp, None, commit_history[0], manifest_file)
 
-        gantry_dataset_obj.pull()
+        gantry_dataset_obj.pull(forced=True)
 
     # try the rollback
     with responses.RequestsMock() as resp:
@@ -582,12 +609,29 @@ def test_rollback(datadir, gantry_dataset_obj, commit_history):
             do_fake_presigned=False,
         )
 
-        gantry_dataset_obj.rollback(original_commit["id"])
+        gantry_dataset_obj.rollback(original_commit["id"], forced=True)
 
         with open(datadir / f"{DATASET_NAME}/.dataset_metadata/HEAD") as f:
             assert json.load(f) == rollback_commit  # verify rollback has been successful
 
 
+def test_rollback_local_changes(datadir, gantry_dataset_obj):
+    """
+    Test trying to pull when local changes exist without forced=True
+    """
+    with open(datadir / "show_diff_dataset/.dataset_metadata/HEAD") as f:
+        original_head = json.load(f)
+
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp)
+        commit = gantry_dataset_obj.rollback("fake-uuid")
+
+        assert commit is None
+
+        with open(datadir / "show_diff_dataset/.dataset_metadata/HEAD") as f:
+            assert original_head == json.load(f)
+
+
 @pytest.mark.parametrize("function", ["commit", "pull", "rollback"])
 def test_function_on_deleted_dataset(datadir, gantry_dataset_obj, function):
     with responses.RequestsMock() as resp:
@@ -722,3 +766,239 @@ def test_add_dataframe(gantry_dataset_obj, filename, expected_filename, commit_m
         )
 
         gantry_dataset_obj.add_dataframe(df, filename, commit_message)
+
+
+def update_manifest_with_checksums(datadir, dataset):
+    """
+    Updates the gantry manifest so that there is no local dif
+    """
+    local_files = get_files_checksum(datadir / dataset.dataset_name)
+
+    with open(dataset._get_file_path(DATASET_MANIFEST_FILE), "w") as gantry_manifest:
+        for file_name, checksum in local_files.items():
+            file_info = {
+                "file_name": file_name,
+                "url": f"{S3_PREFIX}/{DATASET_NAME}/{file_name}",
+                "sha256": checksum,
+                "version_id": "01234567-89ab-cdef-0123-456789abcdef",
+            }
+            gantry_manifest.write(f"{json.dumps(file_info)}\n")
+
+
+def test_stash_files(gantry_dataset_obj, datadir, commit_history):
+    file_content = [{"test": "json"}, {"tomato": "fruit"}, {"oh no": "my file"}]
+    file_names = ["newfile.json", "modfile.json", "delfile.json"]
+    modified_file_content = {"tomato": "vegetable"}
+
+    # prepare files
+    with open(datadir / f"{DATASET_NAME}/{file_names[1]}", "w+") as f:
+        json.dump(file_content[1], f)
+    with open(datadir / f"{DATASET_NAME}/{file_names[2]}", "w+") as f:
+        json.dump(file_content[2], f)
+    # set dataset diff to be none
+    update_manifest_with_checksums(datadir, gantry_dataset_obj)
+
+    # modify files
+    with open(datadir / f"{DATASET_NAME}/{file_names[0]}", "w+") as f:
+        json.dump(file_content[0], f)
+    with open(datadir / f"{DATASET_NAME}/{file_names[1]}", "w+") as f:
+        json.dump(modified_file_content, f)
+    os.remove(datadir / f"{DATASET_NAME}/{file_names[2]}")
+
+    # check that pull fails
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp)
+        assert gantry_dataset_obj.pull() is None
+
+    # stash
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.POST,
+            f"{HOST}/api/v1/datasets/{gantry_dataset_obj._dataset_id}/presign/getobject",
+            json={
+                "response": "ok",
+                "data": {
+                    f"datasets/v1/{DATASET_NAME}/{file_names[1]}": f"{HOST}/file1",
+                    f"datasets/v1/{DATASET_NAME}/{file_names[2]}": f"{HOST}/file2",
+                },
+            },
+            headers={"Content-Type": "application/json"},
+        )
+
+        # adding responses for the fake download urls
+        req_kwargs = {
+            "stream": True,
+        }
+        resp.add(
+            resp.GET,
+            f"{HOST}/file1",
+            body=json.dumps(file_content[1]),
+            match=[matchers.request_kwargs_matcher(req_kwargs)],
+        )
+        resp.add(
+            resp.GET,
+            f"{HOST}/file2",
+            body=json.dumps(file_content[2]),
+            match=[matchers.request_kwargs_matcher(req_kwargs)],
+        )
+
+        gantry_dataset_obj.stash()
+
+    # check that stash was successful
+    # check that relevant files are stashed
+    with open(datadir / f"{DATASET_NAME}/{STASH_FOLDER}/{file_names[0]}") as f:
+        assert json.load(f) == file_content[0]
+    with open(datadir / f"{DATASET_NAME}/{STASH_FOLDER}/{file_names[1]}") as f:
+        assert json.load(f) == modified_file_content
+    # check that the stash file is updated
+    with open(datadir / f"{DATASET_NAME}/{DATASET_STASH_FILE}") as f:
+        assert json.load(f) == {
+            DELETED_FILES: [file_names[2]],
+            MODIFIED_FILES: [file_names[1]],
+            NEW_FILES: [file_names[0]],
+        }
+    # check that the local files are in the correct state
+    assert not os.path.exists(datadir / f"{DATASET_NAME}/{file_names[0]}")
+    for i in [1, 2]:
+        with open(datadir / f"{DATASET_NAME}/{file_names[i]}") as f:
+            assert json.load(f) == file_content[i]
+
+    # check that pull now succeeds
+    manifest_file, commit_json = create_mock_manifest(commit_history)
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp)
+        setup_resp_for_sync(resp, None, commit_json, manifest_file)
+        assert gantry_dataset_obj.pull() == gantry_dataset_obj._prune_commit_info(commit_json)
+
+    # add back file 2 since pulling deletes it
+    with open(datadir / f"{DATASET_NAME}/{file_names[2]}", "w+") as f:
+        json.dump(file_content[2], f)
+
+    # retrieve stash
+    gantry_dataset_obj.restore()
+
+    # check that restore was successful
+    # check that local files are in the correct state
+    with open(datadir / f"{DATASET_NAME}/{file_names[0]}") as f:
+        assert json.load(f) == file_content[0]
+    with open(datadir / f"{DATASET_NAME}/{file_names[1]}") as f:
+        assert json.load(f) == modified_file_content
+    assert not os.path.exists(datadir / f"{DATASET_NAME}/{file_names[2]}")
+    # check stash files are gone
+    assert not os.path.exists(datadir / f"{DATASET_NAME}/{DATASET_STASH_FILE}")
+    for i in [0, 1, 2]:
+        assert not os.path.exists(datadir / f"{DATASET_NAME}/{STASH_FOLDER}/{file_names[i]}")
+
+
+def test_double_stash(gantry_dataset_obj, datadir):
+    file_content = [{"test": "json"}, {"test": "different json"}]
+    file_names = ["newfile.json", "otherfile.json"]
+
+    # set dataset diff to be none
+    update_manifest_with_checksums(datadir, gantry_dataset_obj)
+
+    # modify files and stash
+    with open(datadir / f"{DATASET_NAME}/{file_names[0]}", "w+") as f:
+        json.dump(file_content[0], f)
+    gantry_dataset_obj.stash()
+
+    # check that stash was successful
+    with open(datadir / f"{DATASET_NAME}/{STASH_FOLDER}/{file_names[0]}") as f:
+        assert json.load(f) == file_content[0]
+    with open(datadir / f"{DATASET_NAME}/{DATASET_STASH_FILE}") as f:
+        assert json.load(f) == {NEW_FILES: [file_names[0]], MODIFIED_FILES: [], DELETED_FILES: []}
+    assert not os.path.exists(datadir / f"{DATASET_NAME}/{file_names[0]}")
+
+    # modify files and stash again
+    with open(datadir / f"{DATASET_NAME}/{file_names[1]}", "w+") as f:
+        json.dump(file_content[1], f)
+    gantry_dataset_obj.stash()
+
+    # check that stash was again successful
+    with open(datadir / f"{DATASET_NAME}/{STASH_FOLDER}/{file_names[1]}") as f:
+        assert json.load(f) == file_content[1]
+    with open(datadir / f"{DATASET_NAME}/{DATASET_STASH_FILE}") as f:
+        assert json.load(f) == {NEW_FILES: [file_names[1]], MODIFIED_FILES: [], DELETED_FILES: []}
+    assert not os.path.exists(datadir / f"{DATASET_NAME}/{file_names[1]}")
+
+
+def test_load_nonexistent_stash(gantry_dataset_obj, caplog):
+    with caplog.at_level(logging.WARNING):
+        gantry_dataset_obj.restore()
+    assert "No stashed files were found!" in caplog.text
+
+
+@pytest.mark.parametrize(("function"), [("add_file"), ("add_tabular_file"), ("add_dataframe")])
+def test_add_item_bad_parent(gantry_dataset_obj, function, commit_history):
+    parent_commit_id = uuid.UUID("01234567-89ab-cdef-0123-456789abcdef")
+
+    if function == "add_dataframe":
+        input = pd.DataFrame()
+    else:
+        input = StringIO()
+        input.name = "file.csv"
+
+    with pytest.raises(DatasetHeadOutOfDateException):
+        with responses.RequestsMock() as resp:
+            setup_resp_for_deleted_check(resp)
+            resp.add(
+                resp.GET,
+                f"{HOST}/api/v1/datasets/{DATASET_ID}/commits",
+                json={
+                    "response": "ok",
+                    "data": commit_history,
+                },
+                headers={"Content-Type": "application/json"},
+            )
+
+            if function == "add_file":
+                gantry_dataset_obj.add_file(input, parent_commit_id=parent_commit_id)
+            elif function == "add_tabular_file":
+                gantry_dataset_obj.add_tabular_file(input, parent_commit_id=parent_commit_id)
+            elif function == "add_dataframe":
+                gantry_dataset_obj.add_dataframe(input, parent_commit_id=parent_commit_id)
+
+
+@pytest.mark.parametrize(("function"), [("add_file"), ("add_tabular_file"), ("add_dataframe")])
+def test_add_item_good_parent(gantry_dataset_obj, function, commit_history, add_file_commit):
+    parent_commit_id = commit_history[0]["id"]
+
+    if function == "add_dataframe":
+        input = pd.DataFrame()
+    else:
+        input = StringIO()
+        input.name = "file.csv"
+
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp)
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/datasets/{DATASET_ID}/commits",
+            json={
+                "response": "ok",
+                "data": commit_history,
+            },
+            headers={"Content-Type": "application/json"},
+        )
+        resp.add(
+            resp.POST,
+            f"{HOST}/api/v1/datasets/{DATASET_ID}/file",
+            json={
+                "response": "ok",
+                "data": add_file_commit,
+            },
+            headers={"Content-Type": "application/json"},
+        )
+
+        if function == "add_file":
+            new_commit = gantry_dataset_obj.add_file(input, parent_commit_id=parent_commit_id)
+        elif function == "add_tabular_file":
+            new_commit = gantry_dataset_obj.add_tabular_file(
+                input, parent_commit_id=parent_commit_id
+            )
+        elif function == "add_dataframe":
+            new_commit = gantry_dataset_obj.add_dataframe(input, parent_commit_id=parent_commit_id)
+
+        new_commit["commit_id"] == add_file_commit["id"]
+        new_commit["dataset"] == gantry_dataset_obj.dataset_name
+        new_commit["message"] == "add file test commit"
